;Ultimate Night Light
;January 22, 2011

;Tracks sunrise and sunset times throughout the year to turn on and off
;   the LED night lights.

;Requires time to be set.  Electronics will ride through short power failures.  
;  June 21st has a Sunrise at 4:56 a.m. (MST) and Sunset at 8;02 p.m. (MST)
;  Dec 21st has a Sunrise at 7:48 a.m. (MST) and Sunset at 5:03 p.m. (MST)

;To simplify operations, the clock is set and flashes out Mountain Standard Time
;  throughout the year.

;This project uses a table to look up sunrise and sunset times.  This table is updated every 
;  fifteen days throughout the year.

; Leap year is not ignored, but the date in which the insert of an extra day is not exactly as it occurs.  
;   Just every fourth year, an extra day is inserted.  Overall it's close enough for a town this size.


.include "tn2313def.inc"

; R15 seconds counter

; R16 general purpose temporary use register
; R17 general purpose temporary use register

; R24,R25,R26,R27,R28,R29,R30,R31 = scratch

; R18 is flags register
;	R18,0(1) = AM=0  PM=1
;	R!8,1(2) = Single flash=0  Dbl flash=1 (clock flashout of time)
;	R18,2(4) = seconds counter flag
;	R18,3(8) = 0=display clock   1=no clock display
;	R18,4(16) = minute flag
;	R18,5(32) = Day flag
;	R18,6(64) = Clock Correction flag - 0 = YES correction active  1=Inactive
;	R18,7(128) = r15 clock correction flag

;	r19,0(1) = Fine, variable day correction flag
;	r19,1(2) = Read Out Byte flag 0 = SRAM, 1 = EEPROM

;	r26 = Button Low Byte of total count
;	r27 = Button High Byte of total count


;	EEPROM:
;	$00=	Daily Minutes Correction (coarse)
;	$01=	Fine Minutes Correction determined by interval of $02
;	$02=	Interval for Fine Minutes correction
;	$03 - $33 = Sunrise times (48 entries)
;	$35 - $65 = Sunset times (48 entries)
;	$70 - $71 = Clock Correction Interval Permanent Store ($7C,$7D = SRAM perm store;  $7E,$7F= SRAM variable store)


;*****Initialization


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt

	
START1:			;set up stack pointer and initialize cold start

ldi r16,low(RAMEND)	; Load low byte address of end of RAM into register R16
out	SPL,R16			; Initialize stack pointer to end of internal RAM

ser	r16			;Set to all ones for out port direction as output
out	DDRB,r16	;Set PORTB to output

clr	r16
out	PortB,r16	;LEDs are off when port pins are low

ldi	R16,251		;Set bit 2(4) as input ... remaining bits as output
out	DDRD,R16	;execute data direction with bit 2(4) as input

clr	r16
out	PORTD,R16	;execute to set out ports low, and in port 2(4) with
					; no input resistor

in  r16,TIMSK   ;enable 16-bit match to A register
ori r16,64
out TIMSK,r16

in  r16,TCCR1B
ori r16,11          ; divide by 64 on prescaler
out TCCR1B,r16      ;enable reset of counter to zero (bit3) after match detection.
                    ;and bits 0,1,2 select prescaling source as 64 of clock

                    ;3,579,545 Xtl with 8 division = 447,443
                    ;  divided by 64 = 6991.3
ldi  r16,27          ;comparator high byte for this crystal
out  OCR1AH,r16
ldi  r16,79 		;comparator low byte for this crystal  (clock should run slightly FAST)
					;  clock correction slows clock down at given number of seconds intervals
out  OCR1AL,r16      ;for one second intervals

			
clr	r18				;zero all flags in register r18
clr	r19				;zero all flags in register r19


rcall	DelayOne
ldi		r16,20		;twenty blinks - delay
rcall	BlueLEDFlash


;Following puts clock correction bytes into SRAM

ldi		r16,$70		;EEPROM address
rcall	FetchEEPROM
sts		$78,r16		;put LSD of clock correction interval permanent store low bytes into SRAM permanent store
sts		$7C,r16		;put LSD of clock correction interval variable store into SRAM

ldi		r16,$71		;EEPROM address	
rcall	FetchEEPROM
sts		$79,r16		;put MSD of clock correction interval permanent store low bytes into SRAM permanent store
sts		$7D,r16		;put MSD of clock correction interval variable store into SRAM

ldi		r16,$72		;EEPROM address
rcall	FetchEEPROM
sts		$7A,r16		;put LSD of clock correction interval permanent store high bytes into SRAM permanent store
sts		$7E,r16		;put LSD of clock correction interval permanent store high bytes into SRAM

ldi		r16,$73		;EEPROM address
rcall	FetchEEPROM
sts		$7B,r16		;put MSD of clock correction interval permanent store high bytes into SRAM permanent store
sts		$7F,r16		;put MSD of clock correction interval permanent store high bytes into SRAM 


InitializeClkTime:	;starts clock at 8:00 a.m. on a Sunday

ldi	r16,1
sts	$64,r16
ldi	r16,0
sts	$65,r16			;initialize DOY (day of year) to 1 (January 1st)

ldi	r16,1
sts	$63,r16			;put Sunday(1) in as start of day-of-week

ldi	r16,224			;8:00 A.M. low byte
sts	$61,r16		

ldi	r16,1			;8:00 A.M. high byte
sts	$62,R16

mov	r15,r16			;set seconds counter to one

ldi	r16,2			;set leap year counter to mid point of 2
sts	$6A,r16

rcall	Initialize		;to initialize new values for proper timing

sei					;enable global interrupts


;------------------------------------

MainLoop:

sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService

sbrc	r18,5			;skip if r18,5(32), new day, 24 hour, flag is clear
rcall	SunriseSunset24HrChange

sbrc	r18,3			;skip if r18,3(8) is clear (clock display is on)
rjmp	MainLoop

mov		r16,r15			;get seconds value
cpi		r16,0	
brne	ML2

rjmp	DisplayClock00
rjmp	MainLoop


ML2:

cpi		r16,30
brne	MainLoop

rjmp	DisplayClock30
rjmp	MainLoop


;------------------------------------

ButtonService:		;Decodes button pushes

rcall	ButtonPush

lds		r17,$73		;LSD of total count

cpi		r17,15			
brne	BServ1

rcall	ToggleLEDs
rjmp	BServe8


BServ1:

cpi		r17,16
brne	BServ2

rcall	ToggleClockDisplay
rjmp	BServe8


BServ2:

cpi		r17,17
brne	BServ3

rcall	SetClock


BServ3:				;Two entries -- 1,000's and remaining

cpi		r17,35
brne	BServ4

rcall	SetClockCorrectionInterval
rjmp	BServe8


BServ4:				;Two entries --- Entry 1:  SRAM (1 DIT),  EEPROM (2 Dits)
					;    Entry 2 --- Address for readout

cpi		r17,20
brne	BServe5

rcall	FlashOutByte
rjmp	BServe8


BServe5:			;See instructions.  Several entries are required here

cpi		r17,25
brne	BServe6

rcall	EnterSunriseEEPROMTimes
rjmp	BServe8


BServe6:			;See instructions.  Several entries are required here

cpi		r17,27
brne	BServe8

rcall	EnterSunsetEEPROMTimes
rjmp	BServe8


BServe8:

rcall	OneSecondFlash		;One second flash indicates incorrect entry
rjmp	MainLoop


;-----------------------------------

ToggleLEDs:			;Toggle WHITE LEDs on/off

sbis	PortB,0		;skip if LEDs are on
rjmp	TLED1

cbi		PortB,0		;OFF LEDs
rjmp	TLEDEnd


TLED1:

sbi		PortB,0		;ON LEDs


TLEDEnd:

ret


;------------------------------------


ToggleClockDisplay:		;Toggles clock display on and off 0 = ON

sbrs	r18,3			;skip if r18,3(8) is set (no clock display)
rjmp	TCD1

cbr		r18,8			;clear r18,3(8) - on clock display
rjmp	TCDEnd


TCD1:

sbr		r18,8			;set r18,3(8) - off clock diskplay



TCDEnd:

ret


;-------------------------------------

SetClock:				;Sets DOY (day of year), DOW and clock time - All entries PLUS ONE -- 24 hour format
						;R26,27 pair have LSD, MSD of total count from Button Push

cli						;clear, OFF, interrupts

clr		r30
clr		r31

ldi		r16,4			;give four flashes of Blue LED - four flashes indicate 4 entries
rcall	BlueLEDFlash

						;SET DAY OF YEAR - One plus entry
rcall	StallButtonPush		;Day of Year (HoldDown=50, Dah=10, Dit=1)
sbiw	r26,1			;One less for consistent One Plus Entry
sts		$64,r26
sts		$65,r27			;store new DOY count

rcall	OneBlueLEDFlash


						;SET DAY OF WEEK -- One Plus Entry
rcall	StallButtonPush	;Day of Week entries
dec		r26				;One less for Plus One Entry
sts		$63,r26			;store away day of week entry
rcall	OneBlueLEDFlash


						;SET HOURS
rcall	StallButtonPush
dec		r26				;One Plus entry, thus one less


SClock1:

tst		r26
breq	SClock2

adiw	r30,60
dec		r26
rjmp	SClock1			;converts hours into minutes



SClock2:

rcall	OneBlueLEDflash

						;GET MINUTES
rcall	StallButtonPush
						;Get minutes -- Use PLUS ONE entry (for 12 minutes -- DAH +3 DITS)
						;  (Due to boot up it is not necessary for dec or r16 here)

add		r30,r26
brcc	SClockEnd

inc		r31				;add carry of one


SClockEnd:

cli						;OFF interrupts

sts		$61,r30
sts		$62,r31			;store away new time

clr		r18				;zero all flags
clr		r15				;zero seconds count

sei						;ON interrupts


rcall	Initialize		;to initialize new values for proper timing

ret


;-----------------------------------------

;	LED Flash Ladder

TwoBlueLEDFlash:

ldi		r16,2
rjmp	BlueLEDFlash


OneBlueLEDFlash:		;flashes Blue LED once

ldi		r16,1


BlueLEDFlash:			;flashes Blue LED number of times in r16 (slow count flashes)

tst		r16
breq	BlueEnd

sbi		PortB,1			;ON blue LED
rcall	Delay40
cbi		PortB,1			;OFF blue LED
rcall	DelayHalf		; 1/2 second delay
dec		r16
rjmp	BlueLEDFlash


BlueEnd:

ret


;--------------------------------------------

DeterminePMflag:	;SUBROUTINE - Sets or clears r18,0(1).  1 = PM

lds		r17,$62		;get high byte of minutes
cpi		r17,2		
brlo	DPMa		;less than 2 = AM

cpi		r17,2
brne	DPMb		;PM

lds		r17,$61		;get low byte of minutes
cpi		r17,208
brlo	DPMa		;AM time


DPMb:

sbr		r18,1		;set r18,0(1) as PM
ret


DPMa:

cbr		r18,0		;clear r18,0(1) as AM
ret


;--------------------------------------

DisplayClock00:

rcall	DeterminePMflag
sbrs	r18,0			;skip if r18,0(1) is set (PM)
rjmp	DC00a

ldi		r16,12			;twelve flashes
rcall	SlowIntroFlash	;it is PM, so use slow flash
sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService
rjmp	FlashOutTime


DC00a:

ldi		r16,10			;ten flashes
rcall	FastIntroFlash	;it is AM, so use fast flash
sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService
rjmp	FlashOutTime


;----------------------------------------

DisplayClock30:

rcall	DeterminePMflag
sbrs	r18,0			;skip if r18,0(1) is set (PM)
rjmp	DC30a

ldi		r16,5			;five flashes
rcall	SlowIntroFlash	;it is PM so use slow flash
sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService
rjmp	FlashOutTime


DC30a:

ldi		r16,5			;five flashes
rcall	FastIntroFlash	;it is AM so use fast flash
sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService
rjmp	FlashOutTime


;-------------------------------------

FlashOutTime:

lds		r30,$61			;get current minutes (0-1439)  
lds		r31,$62

cpi		r31,3
brsh	FOTb			;it definitely needs 12 hour subtraction

cpi		r31,2			;if definitely does NOT need 12 hour subtraction
brlo	FOTa

cpi		r30,208			;if lower than 720 minutes then display clock else subtract 12 hours
brlo	FOTa


;---

FOTb:					;12 hour subtraction section

ldi		r16,12			;12 iterations


FOTb1:

sbis	PIND,2
rjmp	ButtonService	;jump out if touch plate touched

sbiw	r30,60			;subtract 60 minutes
dec		r16
brne	FOTb1


;---					;Clock times now in range of 0-719

FOTa:					;this section checks for noon and midnight (12 to 1) flashout correction of 12 flashes

cpi		r31,0
brne	DisplayClock

cpi		r30,60
brsh	DisplayClock


FOTc:					;adds 12 hour to r30,31 pair

ldi		r16,12


FOTc1:

sbis	PIND,2
rjmp	ButtonService	;jump out if touch plate touched

adiw	r30,60
dec		r16
brne	FOTc1

rjmp	DisplayClock


;----------------------------------------

DisplayClock:

lds		r16,$63			;get day of week
cbr		r18,2			;clear r18,1(2) for single flash

rcall	TimeFlashOut	;flash out day of week
sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService

clr		r16				;clear hours counter
mov		r28,r30
mov		r29,r31			;save values in R28,29


DCa:

sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService

inc		r16
mov		r28,r30
mov		r29,r31			;save before subtraction done
sbiw	r30,60			
brcc	DCa				;branch back is carry clear for another round

dec		r16
mov		r30,r28
mov		r31,r29			;restore old values - r16 now has total hours


DCb:

cbr		r18,2			;clear r18,1(2) for single flash

rcall	TimeFlashOut	;flash out hours
sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService

clr		r16				;clear 10 minute segment counter


DCc:

sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService
mov		r17,r30			;save value in r30 in r17 temporarily

inc		r16				;increment 10 min segment counter
subi	r30,10			;get 10 minutes segment count

brcc	DCc

dec		r16
mov		r30,r17			;restore old values to r30 - r16 now has number of 10 minute segments

sbr		r18,2			;set register r18,1(2) for double flash of 10 minute segments

rcall	TimeFlashOut
sbis	PIND,2			;skip if touch plate not touched
rjmp	ButtonService

mov		r16,r30			;put remaining minutes into r16 for flashout

cbr		r18,2			;clear r18,1(2) for single flash

rcall	TimeFlashOut	;flash out minutes.
sbis	PIND,2
rjmp	ButtonService

rjmp	MainLoop


;---------------------------------------------------

TimeFlashOut:   		;SUBROUTINE - r16 has number of flashes --  Only for 12 hour clock (0-719 minutes)

sbrc	r18,1			;skip if r18,1(2) is clear (single flashout)
rjmp	TF2


TF1:

sbis	PIND,2			;skip if touch plate not touched
rjmp	TFEnd

tst		r16
breq	TFEnd

sbi		PortB,1			;ON led
rcall	Escape40Delay
cbi		PortB,1			;OFF led
rcall	EscapeThirdDelay

dec		r16
rjmp	TF1

rjmp	TFEnd


TF2:					;Double flash

sbis	PIND,2			;skip if touch plate not touched
rjmp	TFEnd

tst		r16
breq	TFEnd

sbi		PortB,1			;ON led
rcall 	Escape40Delay
cbi		PortB,1			;OFF led

rcall	EscapeTenthDelay
sbi		PortB,1			;ON led
rcall	Escape40Delay	
cbi		PortB,1			;OFF led
rcall	EscapeThirdDelay

dec		r16
rjmp	TF2


TFEnd:

cbi		PortB,1			;OFF led
rcall	EscapeHalfDelay
rcall	EscapeHalfDelay	;1 second delay

ret


;------------------------------------------

FastIntroFlash:			;SUBROUTINE   r16 contains number of flashes

sbis	PIND,2			;skip if touch plate not touched
ret

sbi		PortB,1			;ON led
rcall	escape40Delay
cbi		PortB,1			;OFF led
rcall	escape20Delay

dec		r16
brne	FastIntroFlash

rcall	escapeOneDelay

ret


;--------------------------------------------
;		;SUBROUTINE - r16 contains number of flashes

SlowIntroFlash:

sbis	PIND,2		;skip if touch plate not touched
ret		

sbi		PortB,1		;ON led
rcall	Escape20Delay
rcall	Escape40Delay
cbi		PortB,1		;OFF led

rcall	Escape20Delay
rcall	Escape40Delay
dec		r16
brne	SlowIntroFlash

rcall	escapeOneDelay

ret


;---------------------------------------------

EndFlash4:			;SUBROUTINE

ldi		r16,6


EndFlash:			;SUBROUTINE

sbi		PortB,1		;ON LED
rcall	Delay40
cbi		PortB,1		;OFF LED
rcall	Delay40
rcall	Delay40		
dec		r16
brne	EndFlash

ret


;---------------------------------------------

IncrementDayOfYear:		;Accounts for leap year (not accurately as to which year). 

push	r30
push	r31

lds		r30,$64
lds		r31,$65			;Load Day of Year (DOY) count

lds		r20,$6A			;get leap year count in r20

adiw	r30,1			;increment day by one

sts		$64,r30
sts		$65,r31			;store new count

cpi		r20,4
breq	IDY1			;it is a leap year

cpi		r31,1			;(it is not a leap year)
brne	IDYend

cpi		r30,110			;is this day =366 ?
brne	IDYend

rjmp	IDY3			;day IS >365


IDY1:					;It is a leap year increment of an extra day

cpi		r31,1
brne	IDYend
cpi		r30,111			;is this day =367 ?
brne	IDYend

clr		r16
sts		$6A,r16			;zero leap year counter and store away


IDY3:

inc		r20				;increment leap year counter
sts		$6A,r20

ldi		r16,1
sts		$64,r16			;set DOY as one (new year count)
clr		r16
sts		$65,r16			;zero high byte of DOY counter


IDYend:

pop		r31
pop		r30

ret


;---------------------------------------------

SunriseSunset24HrChange:	;Changes increment for times of sunrise and sunset. This was determined by
						;   the chart for Salt Lake City, Utah.  All changes are approximate.

						;r20= incremental change temporary store

cbr		r18,32		;clear r18,5(32), New Day, 24 hour flag

rcall	IncrementDayOfYear	;increment day of year including leap year check


;  Following checks for end of year halt for reads from EEPROM.
;    Last Dec. 15th time change will ride out through rest of year.


lds		r16,$64			;load low day of year byte
lds		r17,$65			;load high day of year byte

cpi		r16,1
brne	SS24a

cpi		r17,0
brne	SS24a


;January 1st reset process


ldi		r16,$03			;beginning pointer position for sunrise times
sts		$81,r16			;store in SRAM sunrise pointer location

ldi		r16,$35			;beginning point position for sunset times
sts		$82,r16			;store in SRAM sunset pointer location

clr		r16
sts		$80,r16			;zero fifteen counter

rcall	GrabSunriseSunsetTimes

ret


;-----------------------------------------

;			Subroutine

GrabSunriseSunsetTimes:		;grabs sunrise and sunset times, increments pointers

lds		r16,$81			;get sunrise pointer
mov		r20,r16

rcall	FetchEEPROM		;get sunrise time low byte
sts		$66,r16			;store away new sunrise time low byte

inc		r20
mov		r16,r20

rcall	FetchEEPROM		;get sunrise time high byte
sts		$67,r16			;store away new sunrise time high byte

inc		r20
sts		$81,r20			;store away incremented sunrise pointer value (+2)


lds		r16,$82			;get sunset pointer
mov		r20,r16

rcall	FetchEEPROM		;get sunset time low byte
sts		$68,r16			;store away new sunset time low byte

inc		r20
mov		r16,r20

rcall	FetchEEPROM		;get sunset time high byte
sts		$69,r16			;store away new sunset time high byte

inc		r20
sts		$82,r20			;store away increment sunset pointer value (+2)

ret


;-------------------------------------

SS24a:

cpi		r17,1			;return if past Dec. 21st - no more time change necessary
brne	SS24b

cpi		r16,99
brlo	SS24b

ret


SS24b:

lds		r16,$80

inc		r16
sts		$80,r16

cpi		r16,15
breq	SS24c

ret


SS24c:

clr		r16
sts		$80,r16			;zero fifteen counter

rcall	GrabSunriseSunsetTimes

ret


;----------------------------------------------

	;INTERRUPT ROUTINE


ClockCntDwnInt: 		;interrupt to update clock counters

push	r16
push	r17
push	r28
push	r29
push	r30
push	r31

in		r16,SREG	;save flags
sts		$60,r16

;CLOCK CORRECTION section -- check occurs every new second.  At 30 seconds in on proper interval, correction is made.

;  EEPROM - $70,71,72,73 is permanent storage of clock correction interval

;  SRAM $78,79,7A,7B = permanent store of clock correction interval
;  SRAM $7c,d,e,f = variable store of clock correction interval

;  One second per interval is added to second counter when variable interval reaches zero. (speeds up clock by one second)
;    Thus, length of interval is determined by number of seconds per day according to:  86,400/number of seconds/day desired
;    There are 86,400 seconds per day.   

; Clock Correction Flag Set section

Decrement:

sbrc	r18,6			;skip if r18,6(64) is clear  --  0 = clock correction ON)
rjmp	SubIntervalEnd

lds		r28,$7C
lds		r29,$7D
lds		r30,$7E			
lds		r31,$7F			;get variable clock correction interval counter data

sbiw	r28,1			;decrement count by one
sts		$7C,r28
sts		$7D,r29			;store away new count
brcc	Dec2

sbiw	r30,1			;decrement upper MS pair by one
sts		$7E,r30
sts		$7F,r31			;store away new count


Dec2:

mov		r16,r28
or		r16,r29
or		r16,r30
or		r16,r31			;check for zero count
breq	Dec1

rjmp	DecEnd


Dec1:					;Reset counter to new count and set r19,0(1) add one second flag

sbr		r19,1			;set r19,0(1), add second flag

lds		r16,$78
sts		$7C,r16

lds		r16,$79
sts		$7D,r16

lds		r16,$7A
sts		$7E,r16

lds		r16,$7B
sts		$7F,r16			;put new count in clock correction interval counter from permanent count store


DecEnd:


; END of Clock Correction flag set section --  START of clock correction when flag is set and 15 seconds in
; Following section will check for 15 secoond time and add one second if speed up flag r19,0(1) is set

sbrs	r19,0			;skip if r19,0(1) is set - add one second is needed flag
rjmp	SubIntervalEnd

mov		r16,r15
cpi		r16,15			;Check if 15 seconds into minute			
brne	SubIntervalEnd

cbr		r19,1			;clear r19,0(1) one second addition flag
inc		r15				;ADD one second to r15, seconds counter


SubIntervalEnd:

;End of clock correction area

sbr		r18,4		;set bit r18,2(4) -- second flag 
					;(set each second)
inc		r15			;step one count of SECONDS counter
mov		r16,r15

cpi		r16,60		;check for new minute
breq	CCDI1
rjmp	CCDIexit


CCDI1:				;NEW  MINUTE   

clr		r15			;zero r15 -- SECONDS counter
sbr		r18,16		;set bit 4 (16) in R18 -- the NEW MINUTE flag


;  This section determines when night light LEDS are to be turned on and off

lds		r16,$61
lds		r17,$66		;get time and sunrise time low bytes
cp		r16,r17
brne	OOC1

lds		r16,$62
lds		r17,$67		;get time and sunrise high bytes
cp		r16,r17
brne	OOC1

cbi		PortB,0		;Off LED - sunrise time matches
rjmp	OOCend


OOC1:

lds		r16,$61
lds		r17,$68		;get time and sunset time low bytes
cp		r16,r17
brne	OOCend

lds		r16,$62
lds		r17,$69		;get time and sunset high bytes
cp		r16,r17
brne	OOCend

sbi		PortB,0		;On LED - sunset bytes match


OOCend:

CCDI1Cntr:

lds		r16,$61		;load low minutes byte into R16

inc		r16
sts		$61,r16

brne	CCDI1a		;jump out if (not equal) -- no overflow

lds		r16,$62		;high byte minutes
inc		r16			;increment high byte minutes
sts		$62,r16


CCDI1a:

lds		r16,$62		;check high byte if it is 5
cpi		r16,5

breq	CCDI1b

rjmp	CCDIExit


CCDI1b:			

lds		r16,$61
cpi		r16,160		;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIExit


CCDI1c:			;zero high and low bytes - it is midnight

sbr		r18,32		;Set new day flag, r18,5(32)

clr		r16
sts		$61,r16		;Zero minute counter for midnight reset
sts		$62,r16

lds		r16,$63		;increment day of week counter
inc		r16
sts		$63,r16

cpi		r16,8

brlo	CCDIExit


CCDI1d:

ldi		r16,1		;load with 1 (Sunday)
sts		$63,r16


CCDIExit:		;Exit this interrupt routine

lds		r16,$60		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r31
pop		r30
pop		r29
pop		r28
pop		r17
pop		r16

reti


;-----------------------------

StallButtonPush:	;stalls and waits for button push entry.

sbic	PIND,2		;skip if PIND,2, touch plate is touched
rjmp	StallButtonPush

rjmp	ButtonPush


;-----------------------------------------------------------		

ButtonPush:		;Takes in dit, dah, holddown, and coded input

push	r0		;number of code digits
push	r1		;low code byte
push	r2		;next high code byte
push	r3		;next high code byte
push	r4		;highest code byte
push	r20		;holddown count
push	r21		;dit count
push	r23		;universal counter
push	r24		;combined count
push	r28		;dah count
push	r30		;has total count of entry in Z register, r30,31
push	r31

;r30,r31 (Z register pair contain total value of entry)

clr		r21		;Dit Count
clr		r20		;HoldDown count
clr		r23		;universal counter
clr		r28		;dah count
clr		r0		;clear number of bits in code
clr		r1		;clear lowest code count
clr		r2		;clear next code count
clr		r3		;clear next code count
clr		r4		;clear next code count (clear all code counts)


				;r30,31 has total count

B1:

;inc		r10			;increment r10, random number (not used for ultimate night light)
inc		r23			;universal counter
rcall	Delay40
cpi		r23,60		;delay before first holddown count - 40=one second 
brsh	B3			;branch if - it is a holddown count

sbis	PIND,2		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

inc		r28			;increment dah count

sec					;set carry flag -- it is dah entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,1		;on BLUE LED
rcall	Delay40
rcall	Delay40		;delay for longer on of LEDs
rcall	Delay40
rcall	Delay40
cbi		PortB,1		;off LED
clr		r23
inc		r0			;increment number of code digits


B2d:				;no touch/switch up detection

sbis	PIND,2		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40
cpi		r23,70		;has switch been up 70 counts? 40=one second (1.75 seconds)
brsh	B4Decode1
rjmp	B2d


BPEnd1:

rjmp	BPEnd


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

inc		r21			;increment dit count

clc					;clear carry flag -- it is dit entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,1
rcall	Delay40
cbi		PortB,1

inc		r0			;increment number of code digits (r0)
clr		r23			;clear r23
rjmp	B2d


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
sbi		PortB,1
rcall	Delay40
rcall	Delay40
cbi		PortB,1		;flash RED LED

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40
sbic	PIND,2		;skip if switch is down
rjmp	B4a			;jump out and look for next entry

cpi		r23,30		;pause between continuous hold down of switch
brlo	B4

rjmp	B3


B4a:

clr		r23
rjmp	B2d			;Check for exit or new entry



B4Decode1:

;This section finalizes.  It combines counts

clr		r30
clr		r31			;ready registers for input

mov		r16,r28		;move dah count in r28 into r16


B4comb1:

tst		r16
breq	B4comb2

adiw 	r30,10		;add value of 10 in r30 to r314
dec		r16			
rjmp	B4comb1


B4comb2:

add		r30,r21		;add value in r21 (dit count) to r24
brcs	B4comb2a
rjmp	Bholdown1


B4comb2a:

inc		r31			;carry was set so increment higher byte


Bholdown1:

;Combined value now in Z register pair, r30,31

mov		r16,r20		;put hold down count into r16

Bholdown1b:

tst		r16			;test r20(r16), hold down count
breq	BPEnd
adiw	r30,50		;add fifty count to total
dec		r16
rjmp	Bholdown1b

;Complete combined value now in Z register pair


BPEnd:

;Store away in SRAM $00E7-00EB, Dit,Dah,HolddownCount,LSD Total, MSD Total

sts		$0070,r21		;number of dits
sts		$0071,r28		;number of dahs
sts		$0072,r20		;number of hold down counts
sts		$0073,r30		;LSD of total count
sts		$0074,r31		;MSD of total count

mov		r26,r30
mov		r27,r31			;put low and high bytes of total count into r26 and r27

pop		r31
pop		r30
pop		r28
pop		r24
pop		r23
pop		r21
pop		r20
pop		r4
pop		r3
pop		r2
pop		r1
pop		r0

ret

;  r26 has LSD of total count
;  r27 has MSD of total count


;-------------------------------

;Delay subroutines

;-------------------------

Delay40:

push	r30
push	r31

ldi		r30,231
ldi		r31,10


D40a:

sbiw	r30,1
brne	D40a

pop		r31
pop		r30

ret


;-------------------------------------------

DelayHalf:				;1/2 second delay

push	r16

ldi		r16,20


DHa:

rcall	Delay40
dec		r16
brne	DHa

pop		r16

ret


;--------------------------------------------

DelayOne:

push	r16
ldi		r16,40

rjmp	DHa

 ;-----------------------------------------

 DelayTwo:

 push	r16
 ldi	r16,80
 rjmp	DHa


 ;------------------------------------------


;Escape Delay Routines

;-------------------------------------------


Escape80Delay:		;1/80th second delay.  When touch plate touched, there is quick escape to return

sbis	PIND,2		;skip if touch plate not touched
ret

push	r30
push	r31
ldi		r30,158
ldi		r31,3		;set up delay


E80D2:

sbis	PIND,2		;skip if touch plate not touched
rjmp	E80D1		;exit

sbiw	r30,1		;subtract 1 from r30,31 pair
brne	E80D2


E80D1:

pop		r31
pop		r30
ret


;-------------------------------------------------


Escape40Delay:		;1/40th second delay.

sbis	PIND,2		;skip if touch plate not touched
ret

rcall	Escape80Delay
rcall	Escape80Delay

ret


;---------------------------------------------------

EscapeTenthDelay:

sbis	PIND,2		;skip if touch plate not touched
ret

rcall	Escape40Delay
rcall	Escape40Delay

sbis	PIND,2		;skip if touch plate not touched
ret

rcall	Escape40Delay
rcall	Escape40Delay

ret


;----------------------------------------------------

Escape2TenthDelay:	;.2 second delay

sbis	PIND,2		;skip if touch plate not touched
ret

rcall	EscapeTenthDelay
rcall	EscapeTenthDelay

ret


;-----------------------------------------------------

EscapeThirdDelay:	;1/3 Second Delay

sbis	PIND,2		;skip if touch plate not touched
ret

push	r16
ldi		r16,20

sbis	PIND,2		;skip if touch plate not touched
rjmp	EThird2


EThird1:

rcall	Escape40Delay
dec		r16
brne	EThird1


EThird2:

pop		r16
ret


;-------------------------------------------------------

EscapeHalfDelay:	;Half Second Delay

sbis	PIND,2		;skip if touch plate not touched
ret

push	r16
ldi		r16,20

sbis	PIND,2		;skip if touch plate not touched
rjmp	EHalf2


EHalf1:

rcall	Escape40Delay
dec		r16
brne	EHalf1


EHalf2:

pop		r16
ret


;-------------------------------------------------------

EscapeOneDelay:		;ONE second delay

sbis	PIND,2		;skip if touch plate not touched
ret


push	r16
ldi		r16,40


EOne1:

sbis	PIND,2		;skip if touch plate not touched
rjmp	Eone2

rcall	Escape40Delay
dec		r16
brne	Eone1


EOne2:

pop		r16
ret


;-------------------------------------------------

Escape20Delay:

sbis	PIND,2		;skip if touch plate not touched
ret

rcall	Escape40Delay
rcall	Escape40Delay

ret


;--------------------------------------------------

FetchEEPROM:		;grabs data in permanent saved EEPROM
					;r16 has address location - Then r16 returns data

cli					;clear interrupts

FEE:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE

out		EEAR,r16	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r16,EEDR	;read data from data register

sei					;on interrupts

ret					;code data now in r16


;-------------------------------------------------

WriteEEPROM:		;write to EEPROM - r17 has memory location, r16 has data

cli					;off interrupts


WEE:

sbic	EECR,EEPE	;Wait until ready for write
rjmp	WEE

out		EEAR,r17	;set up address in address register
out		EEDR,r16		;put contents of r16 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

sei					;on interrupts

ret


;--------------------------------------------------

Initialize:			;SUBROUTINE - Initializes sunrise and sunset times and returns to correct day

ldi		r30,109
ldi		r31,1		;set up 365 day count


Initial1:			;go through full year to establish correct position for data reads

rcall	SunriseSunset24HrChange

sbi		PortB,1		;On Blue LED
rcall	Delay40
cbi		PortB,1		;Off Blue LED (Blink light to indicate routine not finished)
rcall	Delay40

sbiw	r30,1		;decrement r30,31 pair

brne	Initial1


ret


;-----------------------------------------

OneSecondFlash:

sbi		PortB,1
rcall	DelayOne
cbi		PortB,1

ret


;-----------------------------------


TogglePort1:

sbis	PortB,1
rjmp	TP4a

cbi		PortB,1
rjmp	TP4b


TP4a:

sbi		PortB,1


TP4b:

rcall	Delay40
ret


;--------------------------

TogglePort5:

sbis	PortB,5
rjmp	TP5a

cbi		PortB,5
rjmp	TP5b


TP5a:

sbi		PortB,5


TP5b:

rcall	Delay40
ret


;--------------------------









FlashOutByte:			;reads out SRAM byte keyed in by ButtonPush  --- followed by next succeeding byte in SRAM
						;  r19,1(2) = 0-SRAM  1-EEPROM

rcall	TwoBlueLEDFlash	;two blue LED flashes for two entries

rcall	StallButtonPush	;One push = read out SRAM, 2 pushes = read out EEPROM
cpi		r26,1
brne	FOB1			


FOB2:					;READ OUT SRAM

rcall	OneBlueLEDFlash

rcall	StallButtonPush	;Enter location to be read out (+1 system)
dec		r26
mov		r30,r26

rcall	DelayTwo
rcall	DelayTwo

ld		r16,z+			;get byte in SRAM pointed to by Z reg pair and and put in r16.  Post increment
rcall	FOBsub

rcall	DelayTwo

ld		r16,z			;get following byte and flash out
rcall	FOBsub

rcall	DelayTwo

ret

;---

FOB1:					;READ OUT EEPROM

rcall	OneBlueLEDFlash

rcall	StallButtonPush
dec		r26				;Plus one entry system 1=0

rcall	DelayTwo
rcall	DelayTwo

mov		r16,r26			;put in address of EEPROM
rcall	FetchEEPROM
rcall	FOBsub

rcall	DelayTwo

inc		r26
mov		r16,r26
rcall	FetchEEPROM
rcall	FOBsub

rcall	DelayTwo

ret


;-------------------------------

FOBsub:

ldi		r17,8


ROBstart:

lsl		r16
brcs	ROB1


sbi		PortB,1			;DIT = 0
rcall	Delay40
rcall	Delay40
cbi		PortB,1
rcall	DelayOne
rjmp	ROB2


ROB1:	;DAH = 1

sbi		PortB,1
rcall	DelayHalf
cbi		PortB,1
rcall	DelayOne


ROB2:

dec		r17
brne	ROBstart

rcall	DelayOne

ret


;----------------------------------

;	Enter times into EEPROM for setup of this program

;Subroutine


EnterSunriseEEPROMTimes:	;#25 Enter Sunrise times into EEPROM table (24 times given every 15th day)
							;   (two bytes/time)

ldi		r16,1
rcall	BlueLEDFlash		;Flash once to enter this routine

ldi		r17,3				;start of EEPROM location for sunrise times (48 entry locations)


EET2:

clr		r20					;clear 48 counter


EET2a:

rcall	StallButtonPush

mov		r16,r26				;save data information into r16 for EEPROM entry

rcall	WriteEEPROM
inc		r17					;increment to next location in EEPROM

sbrs	r17,0				;skip if bit in r17 is set (high byte = two flashes)(odd or even detection)
rjmp	EET4

ldi		r16,1
rcall	SlowIntroFlash

rjmp	EET3



EET4:

ldi		r16,2
rcall	SlowIntroFlash


EET3:

inc		r20					;increment 48 counter
cpi		r20,48				;has limit come?
brne	EET2a


rcall	OneSecondFlash		;to end this section
ret


;--------------------------------------

EnterSunsetEEPROMTimes:		;#27 Enter sunset times into EEPROM table (24 times given every 15th day)
							;   (two bytes/time)

ldi		r16,2				;flash twice to enter this routine
rcall	BlueLEDFlash

ldi 	r17,$35				;start of sunset EEPROM memory section (48 entry locations)

rjmp	EET2


;------------------------------------

SetClockCorrectionInterval:

; This section sets a one second speed up of clock every so many seconds.  To determine the entry put in,
;   divide the total number of seconds per day 86,400 by the number (and fraction) of seconds per day
;   you wish to speed up the clock.  Example, to speed it up 5.73 seoncs per day you would divide 86,400
;	by 5.73, yielding the number -- 15,079.  This is the entry number.  It means that every 15,079 second
;   count the micro-controller will add an extra second.

; To enter, use the first entry for the THOUSANDS unit.  You would enter 15.  But because the PLUG ONE 
;   system is used, you would enter one plus 15 or 16.

; After entry, wait for flash of LED to show acceptance of your first entry, before entering second entry.

; For the second entry, you would then use the hold down count of 50 and dah count of 7 (units of ten) and
;   a dit count of 9.  BUT, because of the Plus one system you would enter 80 instead of 79 or 8 dah counts.


clr		r28
clr		r29
clr		r30
clr		r31				;ZERO counter pairs

sbr		r18,64			;set r18,6(64), clock correction.  1 = Clock correction OFF   0 = clock correction ON

ldi		r16,2			;give two flashes of Blue LED - two flashes indicate 2 entries needed
rcall	BlueLEDFlash

;THOUSANDS entry -- DIT = ZERO --- One plus entry system used so for 18,000, enter 19

rcall	StallButtonPush		;THOUSANDS entry (HoldDown=50, Dah=10, Dit=1)(One Plus entry system for LSD, the Dit)

sbiw	r26,1				;it is a PLUS ONE entry so, one less

mov		r16,r26
or		r16,r27			;check for zero

breq	SCC10


SCC1:					;Thousands Section - add one thousand for each value in r26

mov		r24,r26
mov		r25,r27			;save 1000 entry values in r24,25

cbr		r18,64			;clear clock correction flag, r18,6(64), 0 = ON clock correction


SCC10:					;Enter remaining group after the 1,000 group

ldi		r16,1			;give one flash - last entry
rcall	BlueLEDFlash

rcall	StallButtonPush	;r26,27 have value put in by touch plate
sbiw	r26,1			;reduce by one - Plus One Entry system

mov		r16,r26
or		r16,r27
breq    SCC10c			;bypass setting of flag r18,6(64)

cbr		r18,64			;clear r18,6(64) ON clock correction


SCC10c:

mov		r28,r26
mov		r29,r27			;put low keyed in values into low bytes of counter r28,29,30,31


SCC10a:					;Add 1000 to r28-31 registers for amount in r24,25

mov		r16,r24
or		r16,r25			;check for zero

breq	SCCEnd
rcall	Add1000
						
sbiw	r24,1			;decrement by one, the r24,25 pair (1000 entry data saved earlier)
rjmp	SCC10a


SCCEnd:

sts		$78,r28
sts		$7C,r28
sts		$79,r29
sts		$7D,r29
sts		$7A,r30
sts		$7E,r30
sts		$7B,r31
sts		$7F,r31		;store away new clock correction intervals in SRAM

ldi		r17,$70		;EEPROM address
mov		r16,r28
rcall	WriteEEPROM

ldi		r17,$71		;EEPROM address	
mov		r16,r29
rcall	WriteEEPROM

ldi		r17,$72		;EEPROM address
mov		r16,r30
rcall	WriteEEPROM

ldi		r17,$73		;EEPROM address
mov		r16,r31
rcall	WriteEEPROM

ret


;------------------------------------------

Add1000:			;Adds 1000 to r28-31 registers each time subroutine called

ldi		r16,31


Add1000a:

adiw	r28,32		;Add 32 31 times = 992
brcc	Add1000b

adiw	r30,1			;account for carry


Add1000b:

dec		r16
brne	Add1000a

adiw	r28,8			;add remainder (8) (992+8=1000)
brcc	Add1000Exit

adiw	r30,1			;account for carry


Add1000Exit:

ret


;--------------------------------------------

